@@ -3,7 +3,7 @@
 
 -- |
 -- Module     : Simulation.Aivika.Circuit
--- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2017, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -446,7 +446,7 @@ circuitTransform cir = Transform start
 iterateCircuitInPoints_ :: [Point] -> Circuit a a -> a -> Event ()
 iterateCircuitInPoints_ [] cir a = return ()
 iterateCircuitInPoints_ (p : ps) cir a =
-  enqueueEvent (pointTime p) $
+  enqueueEventWithPriority (pointTime p) (pointPriority p) $
   Event $ \p' ->
   do (a', cir') <- invokeEvent p $ runCircuit cir a
      invokeEvent p $ iterateCircuitInPoints_ ps cir' a'
@@ -457,7 +457,7 @@ iterateCircuitInPoints :: [Point] -> Circuit a a -> a -> Event (Task a)
 iterateCircuitInPoints ps cir a =
   do let loop [] cir a source = triggerSignal source a
          loop (p : ps) cir a source =
-           enqueueEvent (pointTime p) $
+           enqueueEventWithPriority (pointTime p) (pointPriority p) $
            Event $ \p' ->
            do (a', cir') <- invokeEvent p $ runCircuit cir a
               invokeEvent p $ loop ps cir' a' source
@@ -478,7 +478,7 @@ iterateCircuitInIntegTimes_ cir a =
 iterateCircuitInTimes_ :: [Double] -> Circuit a a -> a -> Event ()
 iterateCircuitInTimes_ ts cir a =
   Event $ \p ->
-  do let ps = map (pointAt $ pointRun p) ts
+  do let ps = map (\t -> pointAt (pointRun p) t 0) ts
      invokeEvent p $ 
        iterateCircuitInPoints_ ps cir a 
 
@@ -496,7 +496,7 @@ iterateCircuitInIntegTimes cir a =
 iterateCircuitInTimes :: [Double] -> Circuit a a -> a -> Event (Task a)
 iterateCircuitInTimes ts cir a =
   Event $ \p ->
-  do let ps = map (pointAt $ pointRun p) ts
+  do let ps = map (\t -> pointAt (pointRun p) t 0) ts
      invokeEvent p $ 
        iterateCircuitInPoints ps cir a 
 
@@ -505,7 +505,7 @@ iterateCircuitInTimes ts cir a =
 iterateCircuitInPointsMaybe :: [Point] -> Circuit a (Maybe a) -> a -> Event ()
 iterateCircuitInPointsMaybe [] cir a = return ()
 iterateCircuitInPointsMaybe (p : ps) cir a =
-  enqueueEvent (pointTime p) $
+  enqueueEventWithPriority (pointTime p) (pointPriority p) $
   Event $ \p' ->
   do (a', cir') <- invokeEvent p $ runCircuit cir a
      case a' of
@@ -527,7 +527,7 @@ iterateCircuitInIntegTimesMaybe cir a =
 iterateCircuitInTimesMaybe :: [Double] -> Circuit a (Maybe a) -> a -> Event ()
 iterateCircuitInTimesMaybe ts cir a =
   Event $ \p ->
-  do let ps = map (pointAt $ pointRun p) ts
+  do let ps = map (\t -> pointAt (pointRun p) t 0) ts
      invokeEvent p $ 
        iterateCircuitInPointsMaybe ps cir a
 
@@ -540,7 +540,7 @@ iterateCircuitInPointsEither ps cir a =
   do let loop [] cir ba source = triggerSignal source ba
          loop ps cir ba@(Left b) source = triggerSignal source ba 
          loop (p : ps) cir (Right a) source =
-           enqueueEvent (pointTime p) $
+           enqueueEventWithPriority (pointTime p) (pointPriority p) $
            Event $ \p' ->
            do (ba', cir') <- invokeEvent p $ runCircuit cir a
               invokeEvent p $ loop ps cir' ba' source
@@ -567,7 +567,7 @@ iterateCircuitInIntegTimesEither cir a =
 iterateCircuitInTimesEither :: [Double] -> Circuit a (Either b a) -> a -> Event (Task (Either b a))
 iterateCircuitInTimesEither ts cir a =
   Event $ \p ->
-  do let ps = map (pointAt $ pointRun p) ts
+  do let ps = map (\t -> pointAt (pointRun p) t 0) ts
      invokeEvent p $ 
        iterateCircuitInPointsEither ps cir a
 
