@@ -3,7 +3,7 @@
 
 -- |
 -- Module     : Simulation.Aivika.Generator
--- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2017, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -20,7 +20,11 @@ module Simulation.Aivika.Generator
         newRandomGenerator01) where
 
 import System.Random
+import qualified System.Random.MWC as MWC
 import Data.IORef
+import Data.Word
+import Data.Vector
+import Data.Functor
 
 -- | A discrete probability density function.
 type DiscretePDF a = [(a, Double)]
@@ -71,8 +75,10 @@ data Generator =
               generateWeibull :: Double -> Double -> IO Double,
               -- ^ Generate a random number from the Weibull distribution by
               -- the specified shape and scale.
-              generateDiscrete :: forall a. DiscretePDF a -> IO a
+              generateDiscrete :: forall a. DiscretePDF a -> IO a,
               -- ^ Generate a random value from the specified discrete distribution.
+              generateSequenceNo :: IO Int
+              -- ^ Generate a sequence number which can be considered quite unique.
             }
 
 -- | Generate the uniform random number with the specified minimum and maximum.
@@ -97,9 +103,15 @@ generateUniformInt01 :: IO Double
                         -> IO Int
 generateUniformInt01 g min max =
   do x <- g
-     let min' = fromIntegral min
-         max' = fromIntegral max
-     return $ round (min' + x * (max' - min'))
+     let min' = fromIntegral min - 0.5
+         max' = fromIntegral max + 0.5
+         z    = round (min' + x * (max' - min'))
+         z'   = if z < min
+                then min
+                else if z > max
+                     then max
+                     else z
+     return z'
 
 -- | Generate the triangular random number by the specified minimum, median and maximum.
 generateTriangular01 :: IO Double
@@ -293,7 +305,7 @@ generateDiscrete01 g dpdf =
 -- | Defines a type of the random number generator.
 data GeneratorType = SimpleGenerator
                      -- ^ The simple random number generator.
-                   | SimpleGeneratorWithSeed Int
+                   | SimpleGeneratorWithSeed Word32
                      -- ^ The simple random number generator with the specified seed.
                    | CustomGenerator (IO Generator)
                      -- ^ The custom random number generator.
@@ -306,9 +318,9 @@ newGenerator :: GeneratorType -> IO Generator
 newGenerator tp =
   case tp of
     SimpleGenerator ->
-      newStdGen >>= newRandomGenerator
+      MWC.uniform <$> MWC.createSystemRandom >>= newRandomGenerator01
     SimpleGeneratorWithSeed x ->
-      newRandomGenerator $ mkStdGen x
+      MWC.uniform <$> MWC.initialize (singleton x) >>= newRandomGenerator01
     CustomGenerator g ->
       g
     CustomGenerator01 g ->
@@ -331,6 +343,8 @@ newRandomGenerator01 g =
      g2 <- newNormalGenerator01 g1
      let g3 mu nu = do { x <- g2; return $ mu + nu * x }
          g4 mu nu = do { x <- g2; return $ exp (mu + nu * x) }
+     seqNoRef <- newIORef 0
+     let seqNo = do { x <- readIORef seqNoRef; modifyIORef' seqNoRef (+1); return x }
      return Generator { generateUniform = generateUniform01 g1,
                         generateUniformInt = generateUniformInt01 g1,
                         generateTriangular = generateTriangular01 g1,
@@ -343,4 +357,5 @@ newRandomGenerator01 g =
                         generateGamma = generateGamma01 g2 g1,
                         generateBeta = generateBeta01 g2 g1,
                         generateWeibull = generateWeibull01 g1,
-                        generateDiscrete = generateDiscrete01 g1 }
+                        generateDiscrete = generateDiscrete01 g1,
+                        generateSequenceNo = seqNo }
