@@ -1,7 +1,7 @@
 
 -- |
 -- Module     : Simulation.Aivika.Internal.Cont
--- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2017, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -49,8 +49,10 @@ module Simulation.Aivika.Internal.Cont
         freezeContReentering,
         unfreezeCont,
         substituteCont,
+        substituteContPriority,
         contCanceled,
         contAwait,
+        transferCont,
         traceCont) where
 
 import Data.IORef
@@ -61,6 +63,7 @@ import Data.Monoid
 import Control.Exception
 import Control.Monad
 import Control.Monad.Trans
+import qualified Control.Monad.Catch as MC
 import Control.Applicative
 
 import Debug.Trace
@@ -254,11 +257,10 @@ data ContParamsAux =
   ContParamsAux { contECont :: SomeException -> Event (),
                   contCCont :: () -> Event (),
                   contId :: ContId,
-                  contCancelFlag :: IO Bool,
+                  contCancelRef :: IORef Bool,
                   contCatchFlag  :: Bool }
 
 instance Monad Cont where
-  return  = returnC
   m >>= k = bindC m k
 
 instance ParameterLift Cont where
@@ -277,12 +279,18 @@ instance Functor Cont where
   fmap = liftM
 
 instance Applicative Cont where
-  pure = return
+  pure = returnC
   (<*>) = ap
 
 instance MonadIO Cont where
   liftIO = liftIOC 
 
+instance MC.MonadThrow Cont where
+  throwM = throwCont
+
+instance MC.MonadCatch Cont where
+  catch = catchCont
+
 -- | Invoke the computation.
 invokeCont :: ContParams a -> Cont a -> Event ()
 {-# INLINE invokeCont #-}
@@ -380,7 +388,7 @@ finallyCont (Cont m) (Cont m') =
 -- if it will be wrapped in the 'IO' monad. Therefore, you should use specialised
 -- functions like the stated one that use the 'throw' function but within the 'IO' computation,
 -- which allows already handling the exception.
-throwCont :: IOException -> Cont a
+throwCont :: Exception e => e -> Cont a
 throwCont = liftIO . throw
 
 -- | Run the 'Cont' computation with the specified cancelation source 
@@ -404,7 +412,7 @@ runCont (Cont m) cont econt ccont cid catchFlag =
                    ContParamsAux { contECont = econt,
                                    contCCont = ccont,
                                    contId = cid,
-                                   contCancelFlag = contCancellationActivated cid, 
+                                   contCancelRef = contCancellationActivatedRef cid, 
                                    contCatchFlag  = catchFlag } }
 
 -- | Lift the 'Parameter' computation.
@@ -504,7 +512,7 @@ resumeECont c e =
 -- | Test whether the computation is canceled.
 contCanceled :: ContParams a -> IO Bool
 {-# INLINE contCanceled #-}
-contCanceled c = contCancelFlag $ contAux c
+contCanceled c = readIORef $ contCancelRef $ contAux c
 
 -- | Execute the specified computations in parallel within
 -- the current computation and return their results. The cancellation
@@ -704,7 +712,7 @@ freezeCont :: ContParams a -> Event (FrozenCont a)
 freezeCont c =
   Event $ \p ->
   do rh <- newIORef Nothing
-     rc <- newIORef $ Just c
+     rc <- newIORef $ Just $ substituteContPriority c (pointPriority p)
      h <- invokeEvent p $
           handleSignal (contCancellationInitiating $
                         contId $ contAux c) $ \e ->
@@ -714,7 +722,8 @@ freezeCont c =
                Nothing ->
                  error "The handler was lost: freezeCont."
                Just h ->
-                 do invokeEvent p $ disposeEvent h
+                 do writeIORef rh Nothing
+                    invokeEvent p $ disposeEvent h
                     c <- readIORef rc
                     case c of
                       Nothing -> return ()
@@ -739,7 +748,7 @@ freezeContReentering :: ContParams a -> a -> Event () -> Event (FrozenCont a)
 freezeContReentering c a m =
   Event $ \p ->
   do rh <- newIORef Nothing
-     rc <- newIORef $ Just c
+     rc <- newIORef $ Just $ substituteContPriority c (pointPriority p)
      h <- invokeEvent p $
           handleSignal (contCancellationInitiating $
                         contId $ contAux c) $ \e ->
@@ -749,7 +758,8 @@ freezeContReentering c a m =
                Nothing ->
                  error "The handler was lost: freezeContReentering."
                Just h ->
-                 do invokeEvent p $ disposeEvent h
+                 do writeIORef rh Nothing
+                    invokeEvent p $ disposeEvent h
                     c <- readIORef rc
                     case c of
                       Nothing -> return ()
@@ -769,7 +779,7 @@ freezeContReentering c a m =
           writeIORef rc Nothing
           case c of
             Nothing -> return Nothing
-            z @ (Just c) ->
+            z@(Just c) ->
               do f <- invokeEvent p $
                       contPreemptionBegun $
                       contId $ contAux c
@@ -817,7 +827,8 @@ sleepCont c a =
                 Nothing ->
                   error "The handler was lost: sleepCont."
                 Just h ->
-                  do invokeEvent p $ disposeEvent h
+                  do writeIORef rh Nothing
+                     invokeEvent p $ disposeEvent h
                      case e of
                        ContCancellationInitiating ->
                          invokeEvent p $
@@ -838,28 +849,103 @@ substituteCont :: ContParams a -> (a -> Event ()) -> ContParams a
 {-# INLINE substituteCont #-}
 substituteCont c m = c { contCont = m }
 
+-- | Substitute the continuation priority.
+substituteContPriority :: ContParams a -> EventPriority -> ContParams a
+{-# INLINABLE substituteContPriority #-}
+substituteContPriority c priority = c { contCont = cont,
+                                        contAux  = (contAux c) { contECont = econt,
+                                                                 contCCont = ccont } }
+  where cont a =
+          Event $ \p ->
+          if priority == pointPriority p
+          then invokeEvent p $ contCont c a
+          else invokeEvent p $
+               enqueueEventWithPriority (pointTime p) priority $
+               resumeCont c a
+        econt e =
+          Event $ \p ->
+          if priority == pointPriority p
+          then invokeEvent p $ contECont (contAux c) e
+          else invokeEvent p $
+               enqueueEventWithPriority (pointTime p) priority $
+               resumeECont c e
+        ccont e =
+          Event $ \p ->
+          if priority == pointPriority p
+          then invokeEvent p $ contCCont (contAux c) e
+          else invokeEvent p $
+               enqueueEventWithPriority (pointTime p) priority $
+               contCCont (contAux c) e
+
 -- | Await the signal.
 contAwait :: Signal a -> Cont a
 contAwait signal =
+  Cont $ \c0 ->
+  Event $ \p ->
+  do c <- invokeEvent p $ freezeCont c0
+     r1 <- newIORef Nothing
+     r2 <- newIORef Nothing
+     h1 <- invokeEvent p $
+           handleSignal signal $ 
+           \a -> Event $ 
+                 \p -> do x1 <- readIORef r1
+                          x2 <- readIORef r2
+                          writeIORef r1 Nothing
+                          writeIORef r2 Nothing
+                          case x1 of
+                            Nothing ->
+                              return ()
+                            Just h1 ->
+                              invokeEvent p $ disposeEvent h1
+                          case x2 of
+                            Nothing ->
+                              return ()
+                            Just h2 ->
+                              invokeEvent p $ disposeEvent h2
+                          c <- invokeEvent p $ unfreezeCont c
+                          case c of
+                            Nothing -> return ()
+                            Just c  ->
+                              invokeEvent p $ reenterCont c a
+     h2 <- invokeEvent p $
+           handleSignal (contCancellationInitiating $ contId $ contAux c0) $
+           \a -> Event $
+                 \p -> do x1 <- readIORef r1
+                          x2 <- readIORef r2
+                          writeIORef r1 Nothing
+                          writeIORef r2 Nothing
+                          case x1 of
+                            Nothing ->
+                              return ()
+                            Just h1 ->
+                              invokeEvent p $ disposeEvent h1
+                          case x2 of
+                            Nothing ->
+                              return ()
+                            Just h2 ->
+                              invokeEvent p $ disposeEvent h2
+     writeIORef r1 $ Just h1          
+     writeIORef r2 $ Just h2
+
+-- | Like the GoTo statement it transfers the direction of computation,
+-- but raises an exception when used within 'catchCont' or 'finallyCont'.
+transferCont :: Cont () -> Cont a
+transferCont x =
   Cont $ \c ->
   Event $ \p ->
-  do c <- invokeEvent p $ freezeCont c
-     r <- newIORef Nothing
-     h <- invokeEvent p $
-          handleSignal signal $ 
-          \a -> Event $ 
-                \p -> do x <- readIORef r
-                         case x of
-                           Nothing ->
-                             error "The signal was lost: contAwait."
-                           Just x ->
-                             do invokeEvent p $ disposeEvent x
-                                c <- invokeEvent p $ unfreezeCont c
-                                case c of
-                                  Nothing -> return ()
-                                  Just c  ->
-                                    invokeEvent p $ reenterCont c a
-     writeIORef r $ Just h          
+  do let worker =
+           do let cid   = contId $ contAux c
+                  cont  = return
+                  econt = throwEvent
+                  ccont = return
+              when (contCatchFlag $ contAux c) $
+                error "Cannot be combined with the exception handling: unsafeTransferCont"
+              invokeEvent p $
+                runCont x cont econt ccont cid False
+     z <- contCanceled c
+     if z
+       then cancelCont p c
+       else worker
 
 -- | Show the debug message with the current simulation time.
 traceCont :: String -> Cont a -> Cont a
