@@ -1,9 +1,9 @@
 
-{-# LANGUAGE RecursiveDo #-}
+{-# LANGUAGE RecursiveDo, RankNTypes #-}
 
 -- |
 -- Module     : Simulation.Aivika.Internal.Dynamics
--- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2017, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -39,6 +39,8 @@ import Control.Exception
 import Control.Monad
 import Control.Monad.Trans
 import Control.Monad.Fix
+import Control.Monad.Fail
+import qualified Control.Monad.Catch as MC
 import Control.Applicative
 
 import Debug.Trace
@@ -51,7 +53,6 @@ import Simulation.Aivika.Internal.Simulation
 newtype Dynamics a = Dynamics (Point -> IO a)
 
 instance Monad Dynamics where
-  return  = returnD
   m >>= k = bindD m k
 
 returnD :: a -> Dynamics a
@@ -74,7 +75,7 @@ runDynamicsInStartTime (Dynamics m) =
 -- | Run the 'Dynamics' computation in the final time point.
 runDynamicsInStopTime :: Dynamics a -> Simulation a
 runDynamicsInStopTime (Dynamics m) =
-  Simulation $ m . integStopPoint
+  Simulation $ m . simulationStopPoint
 
 -- | Run the 'Dynamics' computation in all integration time points.
 runDynamicsInIntegTimes :: Dynamics a -> Simulation [IO a]
@@ -84,20 +85,23 @@ runDynamicsInIntegTimes (Dynamics m) =
 -- | Run the 'Dynamics' computation in the specified time point.
 runDynamicsInTime :: Double -> Dynamics a -> Simulation a
 runDynamicsInTime t (Dynamics m) =
-  Simulation $ \r -> m $ pointAt r t
+  Simulation $ \r -> m $ pointAt r t 0
 
 -- | Run the 'Dynamics' computation in the specified time points.
 runDynamicsInTimes :: [Double] -> Dynamics a -> Simulation [IO a]
 runDynamicsInTimes ts (Dynamics m) =
-  Simulation $ \r -> return $ map (m . pointAt r) ts 
+  Simulation $ \r -> return $ map (\t -> m $ pointAt r t 0) ts 
 
 instance Functor Dynamics where
   fmap = liftMD
 
 instance Applicative Dynamics where
-  pure = return
+  pure = returnD
   (<*>) = ap
 
+instance MonadFail Dynamics where
+  fail = error
+
 instance Eq (Dynamics a) where
   x == y = error "Can't compare dynamics." 
 
@@ -192,6 +196,34 @@ finallyDynamics (Dynamics m) (Dynamics m') =
 throwDynamics :: Exception e => e -> Dynamics a
 throwDynamics = throw
 
+-- | Runs an action with asynchronous exceptions disabled.
+maskDynamics :: ((forall a. Dynamics a -> Dynamics a) -> Dynamics b) -> Dynamics b
+maskDynamics a =
+  Dynamics $ \p ->
+  MC.mask $ \u ->
+  invokeDynamics p (a $ q u)
+  where q u (Dynamics b) = Dynamics (u . b)
+
+-- | Like 'maskDynamics', but the masked computation is not interruptible.
+uninterruptibleMaskDynamics :: ((forall a. Dynamics a -> Dynamics a) -> Dynamics b) -> Dynamics b
+uninterruptibleMaskDynamics a =
+  Dynamics $ \p ->
+  MC.uninterruptibleMask $ \u ->
+  invokeDynamics p (a $ q u)
+  where q u (Dynamics b) = Dynamics (u . b)
+
+-- | An implementation of 'generalBracket'.
+generalBracketDynamics :: Dynamics a
+                          -> (a -> MC.ExitCase b -> Dynamics c)
+                          -> (a -> Dynamics b)
+                          -> Dynamics (b, c)
+generalBracketDynamics acquire release use =
+  Dynamics $ \p -> do
+    MC.generalBracket
+      (invokeDynamics p acquire)
+      (\resource e -> invokeDynamics p $ release resource e)
+      (\resource -> invokeDynamics p $ use resource)
+
 -- | Invoke the 'Dynamics' computation.
 invokeDynamics :: Point -> Dynamics a -> IO a
 {-# INLINE invokeDynamics #-}
@@ -202,6 +234,17 @@ instance MonadFix Dynamics where
     Dynamics $ \p ->
     do { rec { a <- invokeDynamics p (f a) }; return a }
 
+instance MC.MonadThrow Dynamics where
+  throwM = throwDynamics
+
+instance MC.MonadCatch Dynamics where
+  catch = catchDynamics
+
+instance MC.MonadMask Dynamics where
+  mask = maskDynamics
+  uninterruptibleMask = uninterruptibleMaskDynamics
+  generalBracket = generalBracketDynamics
+
 -- | Computation that returns the current simulation time.
 time :: Dynamics Double
 time = Dynamics $ return . pointTime 
