@@ -1,9 +1,9 @@
 
-{-# LANGUAGE RecursiveDo #-}
+{-# LANGUAGE RecursiveDo, RankNTypes #-}
 
 -- |
 -- Module     : Simulation.Aivika.Internal.Event
--- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2017, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -31,7 +31,9 @@ module Simulation.Aivika.Internal.Event
         runEventInStartTime,
         runEventInStopTime,
         -- * Event Queue
+        EventPriority(..),
         enqueueEvent,
+        enqueueEventWithPriority,
         enqueueEventWithCancellation,
         enqueueEventWithStartTime,
         enqueueEventWithStopTime,
@@ -40,6 +42,7 @@ module Simulation.Aivika.Internal.Event
         enqueueEventWithIntegTimes,
         yieldEvent,
         eventQueueCount,
+        eventPriority,
         -- * Cancelling Event
         EventCancellation,
         cancelEvent,
@@ -60,17 +63,20 @@ module Simulation.Aivika.Internal.Event
         traceEvent) where
 
 import Data.IORef
-import Data.Monoid
+import Data.Monoid hiding ((<>))
+import Data.Semigroup (Semigroup(..))
 
 import Control.Exception
 import Control.Monad
 import Control.Monad.Trans
 import Control.Monad.Fix
+import Control.Monad.Fail
+import qualified Control.Monad.Catch as MC
 import Control.Applicative
 
 import Debug.Trace (trace)
 
-import qualified Simulation.Aivika.PriorityQueue as PQ
+import qualified Simulation.Aivika.PriorityQueue.EventQueue as PQ
 
 import Simulation.Aivika.Internal.Specs
 import Simulation.Aivika.Internal.Parameter
@@ -82,7 +88,6 @@ import Simulation.Aivika.Internal.Dynamics
 newtype Event a = Event (Point -> IO a)
 
 instance Monad Event where
-  return  = returnE
   m >>= k = bindE m k
 
 returnE :: a -> Event a
@@ -101,9 +106,12 @@ instance Functor Event where
   fmap = liftME
 
 instance Applicative Event where
-  pure = return
+  pure = returnE
   (<*>) = ap
 
+instance MonadFail Event where
+  fail = error
+
 liftME :: (a -> b) -> Event a -> Event b
 {-# INLINE liftME #-}
 liftME f (Event x) =
@@ -162,6 +170,34 @@ finallyEvent (Event m) (Event m') =
 throwEvent :: Exception e => e -> Event a
 throwEvent = throw
 
+-- | Runs an action with asynchronous exceptions disabled.
+maskEvent :: ((forall a. Event a -> Event a) -> Event b) -> Event b
+maskEvent a =
+  Event $ \p ->
+  MC.mask $ \u ->
+  invokeEvent p (a $ q u)
+  where q u (Event b) = Event (u . b)
+
+-- | Like 'maskEvent', but the masked computation is not interruptible.
+uninterruptibleMaskEvent :: ((forall a. Event a -> Event a) -> Event b) -> Event b
+uninterruptibleMaskEvent a =
+  Event $ \p ->
+  MC.uninterruptibleMask $ \u ->
+  invokeEvent p (a $ q u)
+  where q u (Event b) = Event (u . b)
+
+-- | An implementation of 'generalBracket'.
+generalBracketEvent :: Event a
+                       -> (a -> MC.ExitCase b -> Event c)
+                       -> (a -> Event b)
+                       -> Event (b, c)
+generalBracketEvent acquire release use =
+  Event $ \p -> do
+    MC.generalBracket
+      (invokeEvent p acquire)
+      (\resource e -> invokeEvent p $ release resource e)
+      (\resource -> invokeEvent p $ use resource)
+
 -- | Invoke the 'Event' computation.
 invokeEvent :: Point -> Event a -> IO a
 {-# INLINE invokeEvent #-}
@@ -172,6 +208,17 @@ instance MonadFix Event where
     Event $ \p ->
     do { rec { a <- invokeEvent p (f a) }; return a }
 
+instance MC.MonadThrow Event where
+  throwM = throwEvent
+
+instance MC.MonadCatch Event where
+  catch = catchEvent
+
+instance MC.MonadMask Event where
+  mask = maskEvent
+  uninterruptibleMask = uninterruptibleMaskEvent
+  generalBracket = generalBracketEvent
+
 -- | Defines how the events are processed.
 data EventProcessing = CurrentEvents
                        -- ^ either process all earlier and then current events,
@@ -201,7 +248,15 @@ enqueueEvent :: Double -> Event () -> Event ()
 enqueueEvent t (Event m) =
   Event $ \p ->
   let pq = queuePQ $ runEventQueue $ pointRun p
-  in PQ.enqueue pq t m
+  in PQ.enqueue pq t (pointPriority p) m
+
+-- | Enqueue the event which must be actuated at the specified time
+-- given the priority.
+enqueueEventWithPriority :: Double -> EventPriority -> Event () -> Event ()
+enqueueEventWithPriority t priority (Event m) =
+  Event $ \p ->
+  let pq = queuePQ $ runEventQueue $ pointRun p
+  in PQ.enqueue pq t priority m
 
 -- | Process the pending events.
 processPendingEventsCore :: Bool -> Dynamics ()
@@ -219,7 +274,7 @@ processPendingEventsCore includingCurrentEvents = Dynamics r where
            r  = pointRun p
        f <- PQ.queueNull pq
        unless f $
-         do (t2, c2) <- PQ.queueFront pq
+         do (t2, priority2, c2) <- PQ.queueFront pq
             let t = queueTime q
             t' <- readIORef t
             when (t2 < t') $ 
@@ -233,6 +288,7 @@ processPendingEventsCore includingCurrentEvents = Dynamics r where
                      dt = spcDT sc
                      n2 = fromIntegral $ floor ((t2 - t0) / dt)
                  c2 $ p { pointTime = t2,
+                          pointPriority = priority2,
                           pointIteration = n2,
                           pointPhase = -1 }
                  call q p
@@ -301,6 +357,11 @@ eventQueueCount :: Event Int
 eventQueueCount =
   Event $ PQ.queueCount . queuePQ . runEventQueue . pointRun
 
+-- | Return the current event priority.
+eventPriority :: Event EventPriority
+eventPriority =
+  Event $ return . pointPriority
+
 -- | Actuate the event handler in the specified time points.
 enqueueEventWithTimes :: [Double] -> Event () -> Event ()
 enqueueEventWithTimes ts e = loop ts
@@ -334,7 +395,7 @@ enqueueEventWithStartTime e =
 enqueueEventWithStopTime :: Event () -> Event ()
 enqueueEventWithStopTime e =
   Event $ \p ->
-  let p0 = integStopPoint $ pointRun p
+  let p0 = simulationStopPoint $ pointRun p
   in invokeEvent p $ enqueueEventWithPoints [p0] e
 
 -- | It allows cancelling the event.
@@ -423,10 +484,13 @@ newtype DisposableEvent =
                     -- ^ Dispose something within the 'Event' computation.
                   }
 
+instance Semigroup DisposableEvent where
+  DisposableEvent x <> DisposableEvent y = DisposableEvent $ x >> y
+
 instance Monoid DisposableEvent where
 
   mempty = DisposableEvent $ return ()
-  mappend (DisposableEvent x) (DisposableEvent y) = DisposableEvent $ x >> y
+  mappend = (<>)
 
 -- | Retry the current computation as possible, using the specified argument
 -- as a 'SimulationRetry' exception message in case of failure.
