@@ -1,9 +1,9 @@
 
-{-# LANGUAGE RecursiveDo #-}
+{-# LANGUAGE RecursiveDo, RankNTypes #-}
 
 -- |
 -- Module     : Simulation.Aivika.Internal.Parameter
--- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2017, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -47,6 +47,8 @@ import Control.Concurrent.MVar
 import Control.Monad
 import Control.Monad.Trans
 import Control.Monad.Fix
+import Control.Monad.Fail
+import qualified Control.Monad.Catch as MC
 import Control.Applicative
 
 import Data.IORef
@@ -64,7 +66,6 @@ import Simulation.Aivika.Internal.Specs
 newtype Parameter a = Parameter (Run -> IO a)
 
 instance Monad Parameter where
-  return  = returnP
   m >>= k = bindP m k
 
 returnP :: a -> Parameter a
@@ -122,9 +123,12 @@ instance Functor Parameter where
   fmap = liftMP
 
 instance Applicative Parameter where
-  pure = return
+  pure = returnP
   (<*>) = ap
 
+instance MonadFail Parameter where
+  fail = error
+
 instance Eq (Parameter a) where
   x == y = error "Can't compare parameters." 
 
@@ -203,6 +207,34 @@ finallyParameter (Parameter m) (Parameter m') =
 throwParameter :: Exception e => e -> Parameter a
 throwParameter = throw
 
+-- | Runs an action with asynchronous exceptions disabled.
+maskParameter :: ((forall a. Parameter a -> Parameter a) -> Parameter b) -> Parameter b
+maskParameter a =
+  Parameter $ \r ->
+  MC.mask $ \u ->
+  invokeParameter r (a $ q u)
+  where q u (Parameter b) = Parameter (u . b)
+
+-- | Like 'maskParameter', but the masked computation is not interruptible.
+uninterruptibleMaskParameter :: ((forall a. Parameter a -> Parameter a) -> Parameter b) -> Parameter b
+uninterruptibleMaskParameter a =
+  Parameter $ \r ->
+  MC.uninterruptibleMask $ \u ->
+  invokeParameter r (a $ q u)
+  where q u (Parameter b) = Parameter (u . b)
+
+-- | An implementation of 'generalBracket'.
+generalBracketParameter :: Parameter a
+                           -> (a -> MC.ExitCase b -> Parameter c)
+                           -> (a -> Parameter b)
+                           -> Parameter (b, c)
+generalBracketParameter acquire release use =
+  Parameter $ \r -> do
+    MC.generalBracket
+      (invokeParameter r acquire)
+      (\resource e -> invokeParameter r $ release resource e)
+      (\resource -> invokeParameter r $ use resource)
+
 -- | Invoke the 'Parameter' computation.
 invokeParameter :: Run -> Parameter a -> IO a
 {-# INLINE invokeParameter #-}
@@ -213,6 +245,17 @@ instance MonadFix Parameter where
     Parameter $ \r ->
     do { rec { a <- invokeParameter r (f a) }; return a }  
 
+instance MC.MonadThrow Parameter where
+  throwM = throwParameter
+
+instance MC.MonadCatch Parameter where
+  catch = catchParameter
+
+instance MC.MonadMask Parameter where
+  mask = maskParameter
+  uninterruptibleMask = uninterruptibleMaskParameter
+  generalBracket = generalBracketParameter
+  
 -- | Memoize the 'Parameter' computation, always returning the same value
 -- within a simulation run. However, the value will be recalculated for other
 -- simulation runs. Also it is thread-safe when different simulation runs
