@@ -1,7 +1,7 @@
 
 -- |
 -- Module     : Simulation.Aivika.Internal.Process
--- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2017, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -51,9 +51,12 @@ module Simulation.Aivika.Internal.Process
         holdProcess,
         interruptProcess,
         processInterrupted,
+        processInterruptionTime,
         passivateProcess,
+        passivateProcessBefore,
         processPassive,
         reactivateProcess,
+        reactivateProcessImmediately,
         cancelProcessWithId,
         cancelProcess,
         processCancelled,
@@ -80,6 +83,8 @@ module Simulation.Aivika.Internal.Process
         catchProcess,
         finallyProcess,
         throwProcess,
+        -- * Process Priority
+        processWithPriority,
         -- * Utilities
         zipProcessParallel,
         zip3ProcessParallel,
@@ -90,6 +95,8 @@ module Simulation.Aivika.Internal.Process
         neverProcess,
         -- * Retrying Computation
         retryProcess,
+        -- * GoTo Statement
+        transferProcess,
         -- * Debugging
         traceProcess) where
 
@@ -99,6 +106,8 @@ import Data.IORef
 import Control.Exception
 import Control.Monad
 import Control.Monad.Trans
+import Control.Monad.Fail
+import qualified Control.Monad.Catch as MC
 import Control.Applicative
 
 import Simulation.Aivika.Internal.Specs
@@ -112,12 +121,14 @@ import Simulation.Aivika.Signal
 -- | Represents a process identifier.
 data ProcessId = 
   ProcessId { processStarted :: IORef Bool,
-              processReactCont     :: IORef (Maybe (ContParams ())), 
+              processReactCont     :: IORef (Maybe (ContParams ())),
+              processReactPriority :: IORef Int,
               processContId  :: ContId,
               processInterruptRef  :: IORef Bool, 
               processInterruptCont :: IORef (Maybe (ContParams ())),
               processInterruptTime :: IORef Double,
-              processInterruptVersion :: IORef Int }
+              processInterruptVersion :: IORef Int,
+              processInterruptPriority :: IORef Int }
 
 -- | Specifies a discontinuous process that can suspend at any time
 -- and then resume later.
@@ -150,6 +161,7 @@ holdProcess dt =
      writeIORef x $ Just c
      writeIORef (processInterruptRef pid) False
      writeIORef (processInterruptTime pid) t
+     writeIORef (processInterruptPriority pid) (pointPriority p)
      v <- readIORef (processInterruptVersion pid)
      invokeEvent p $
        enqueueEvent t $
@@ -169,10 +181,13 @@ interruptProcess pid =
      case a of
        Nothing -> return ()
        Just c ->
-         do writeIORef x Nothing
+         do priority <- readIORef (processInterruptPriority pid)
+            writeIORef x Nothing
             writeIORef (processInterruptRef pid) True
             modifyIORef (processInterruptVersion pid) $ (+) 1
-            invokeEvent p $ enqueueEvent (pointTime p) $ resumeCont c ()
+            invokeEvent p $
+              enqueueEventWithPriority (pointTime p) priority $
+              resumeCont c ()
             
 -- | Test whether the process with the specified identifier was interrupted.
 processInterrupted :: ProcessId -> Event Bool
@@ -180,6 +195,20 @@ processInterrupted pid =
   Event $ \p ->
   readIORef (processInterruptRef pid)
 
+-- | Return the expected interruption time after finishing the 'holdProcess' computation,
+-- which value may change if the corresponding process is preempted.
+processInterruptionTime :: ProcessId -> Event (Maybe Double)
+processInterruptionTime pid =
+  Event $ \p ->
+  do let x = processInterruptCont pid
+     a <- readIORef x
+     case a of
+       Just c  ->
+         do t <- readIORef (processInterruptTime pid)
+            return (Just t)
+       Nothing ->
+         return Nothing
+
 -- | Define a reaction when the process with the specified identifier is preempted.
 processPreempted :: ProcessId -> Event ()
 processPreempted pid =
@@ -188,7 +217,8 @@ processPreempted pid =
      a <- readIORef x
      case a of
        Just c ->
-         do writeIORef x Nothing
+         do priority <- readIORef (processInterruptPriority pid)
+            writeIORef x Nothing
             writeIORef (processInterruptRef pid) True
             modifyIORef (processInterruptVersion pid) $ (+) 1
             t <- readIORef (processInterruptTime pid)
@@ -198,7 +228,7 @@ processPreempted pid =
                   invokeEvent p $
                   invokeCont c $
                   invokeProcess pid $
-                  holdProcess dt
+                  processWithPriority priority >> holdProcess dt
             invokeEvent p $
               reenterCont c' ()
        Nothing ->
@@ -220,8 +250,27 @@ passivateProcess =
   do let x = processReactCont pid
      a <- readIORef x
      case a of
-       Nothing -> writeIORef x $ Just c
-       Just _  -> error "Cannot passivate the process twice: passivateProcess"
+       Nothing ->
+         do writeIORef x $ Just c
+            writeIORef (processReactPriority pid) (pointPriority p)
+       Just _  ->
+         error "Cannot passivate the process twice: passivateProcess"
+
+-- | Passivate the process before performing some action.
+passivateProcessBefore :: Event () -> Process ()
+passivateProcessBefore m =
+  Process $ \pid ->
+  Cont $ \c ->
+  Event $ \p ->
+  do let x = processReactCont pid
+     a <- readIORef x
+     case a of
+       Nothing ->
+         do writeIORef x $ Just c
+            writeIORef (processReactPriority pid) (pointPriority p)
+            invokeEvent p m
+       Just _  ->
+         error "Cannot passivate the process twice: passivateProcessBefore"
 
 -- | Test whether the process with the specified identifier is passivated.
 processPassive :: ProcessId -> Event Bool
@@ -241,8 +290,29 @@ reactivateProcess pid =
        Nothing -> 
          return ()
        Just c ->
-         do writeIORef x Nothing
-            invokeEvent p $ enqueueEvent (pointTime p) $ resumeCont c ()
+         do priority <- readIORef (processReactPriority pid)
+            writeIORef x Nothing
+            invokeEvent p $
+              enqueueEventWithPriority (pointTime p) priority $
+              resumeCont c ()
+
+-- | Reactivate a process with the specified identifier immediately.
+reactivateProcessImmediately :: ProcessId -> Event ()
+reactivateProcessImmediately pid =
+  Event $ \p ->
+  do let x = processReactCont pid
+     a <- readIORef x
+     case a of
+       Nothing -> 
+         return ()
+       Just c ->
+         do priority <- readIORef (processReactPriority pid)
+            writeIORef x Nothing
+            if priority == pointPriority p
+              then invokeEvent p $ resumeCont c ()
+              else invokeEvent p $
+                   enqueueEventWithPriority (pointTime p) priority $
+                   resumeCont c ()
 
 -- | Prepare the processes identifier for running.
 processIdPrepare :: ProcessId -> Event ()
@@ -343,13 +413,17 @@ newProcessId =
      z <- liftIO $ newIORef Nothing
      t <- liftIO $ newIORef 0
      v <- liftIO $ newIORef 0
+     priority1 <- liftIO $ newIORef 0
+     priority2 <- liftIO $ newIORef 0
      return ProcessId { processStarted = y,
-                        processReactCont     = x, 
+                        processReactCont     = x,
+                        processReactPriority = priority1,
                         processContId  = c, 
                         processInterruptRef  = i,
                         processInterruptCont = z,
                         processInterruptTime = t,
-                        processInterruptVersion = v }
+                        processInterruptVersion = v,
+                        processInterruptPriority = priority2 }
 
 -- | Cancel a process with the specified identifier, interrupting it if needed.
 cancelProcessWithId :: ProcessId -> Event ()
@@ -399,16 +473,18 @@ instance Eq ProcessId where
   x == y = processReactCont x == processReactCont y    -- for the references are unique
 
 instance Monad Process where
-  return  = returnP
   m >>= k = bindP m k
 
 instance Functor Process where
   fmap = liftM
 
 instance Applicative Process where
-  pure = return
+  pure = returnP
   (<*>) = ap
 
+instance MonadFail Process where
+  fail = error
+
 instance ParameterLift Process where
   liftParameter = liftPP
 
@@ -424,6 +500,12 @@ instance EventLift Process where
 instance MonadIO Process where
   liftIO = liftIOP
   
+instance MC.MonadThrow Process where
+  throwM = throwProcess
+
+instance MC.MonadCatch Process where
+  catch = catchProcess
+
 returnP :: a -> Process a
 {-# INLINE returnP #-}
 returnP a = Process $ \pid -> return a
@@ -479,6 +561,18 @@ finallyProcess (Process m) (Process m') =
 throwProcess :: Exception e => e -> Process a
 throwProcess = liftIO . throw
 
+-- | Proceed with the process that would use the specified event priority.
+processWithPriority :: EventPriority -> Process ()
+processWithPriority priority =
+  Process $ \pid ->
+  Cont $ \c ->
+  Event $ \p ->
+  if priority == pointPriority p
+  then invokeEvent p $ resumeCont c ()
+  else invokeEvent p $
+       enqueueEventWithPriority (pointTime p) priority $
+       resumeCont c ()
+
 -- | Execute the specified computations in parallel within
 -- the current computation and return their results. The cancellation
 -- of any of the nested computations affects the current computation.
@@ -655,6 +749,10 @@ unzipProcess xy =
 --
 -- A cancellation of the child process doesn't lead to cancelling the parent process.
 -- Then 'Nothing' is returned within the computation.
+--
+-- This is a heavy-weight operation destined for working with arbitrary discontinuous
+-- processes. Please consider using a more light-weight function 'interruptProcess' or else
+-- 'cancelProcessWithId' whenever possible.
 timeoutProcess :: Double -> Process a -> Process (Maybe a)
 timeoutProcess timeout p =
   do pid <- liftSimulation newProcessId
@@ -670,6 +768,10 @@ timeoutProcess timeout p =
 --
 -- A cancellation of the child process doesn't lead to cancelling the parent process.
 -- Then 'Nothing' is returned within the computation.
+--
+-- This is a heavy-weight operation destined for working with arbitrary discontinuous
+-- processes. Please consider using a more light-weight function 'interruptProcess' or else
+-- 'cancelProcessWithId' whenever possible.
 timeoutProcessUsingId :: Double -> ProcessId -> Process a -> Process (Maybe a)
 timeoutProcessUsingId timeout pid p =
   do s <- liftSimulation newSignalSource
@@ -724,6 +826,12 @@ neverProcess =
 retryProcess :: String -> Process a
 retryProcess = liftEvent . retryEvent
 
+-- | Like the GoTo statement it transfers the direction of computation,
+-- but raises an exception when used within 'catchProcess' or 'finallyProcess'.
+transferProcess :: Process () -> Process a
+transferProcess (Process m) =
+  Process $ \pid -> transferCont (m pid)
+
 -- | Show the debug message with the current simulation time.
 traceProcess :: String -> Process a -> Process a
 traceProcess message m =
