@@ -1,9 +1,9 @@
 
-{-# LANGUAGE RecursiveDo, ExistentialQuantification, DeriveDataTypeable #-}
+{-# LANGUAGE RecursiveDo, ExistentialQuantification, DeriveDataTypeable, RankNTypes #-}
 
 -- |
 -- Module     : Simulation.Aivika.Internal.Simulation
--- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2017, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -39,6 +39,8 @@ import Control.Exception
 import Control.Monad
 import Control.Monad.Trans
 import Control.Monad.Fix
+import Control.Monad.Fail
+import qualified Control.Monad.Catch as MC
 import Control.Applicative
 
 import Data.IORef
@@ -53,7 +55,6 @@ import Simulation.Aivika.Internal.Parameter
 newtype Simulation a = Simulation (Run -> IO a)
 
 instance Monad Simulation where
-  return  = returnS
   m >>= k = bindS m k
 
 returnS :: a -> Simulation a
@@ -118,9 +119,12 @@ instance Functor Simulation where
   fmap = liftMS
 
 instance Applicative Simulation where
-  pure = return
+  pure = returnS
   (<*>) = ap
 
+instance MonadFail Simulation where
+  fail = error
+
 liftMS :: (a -> b) -> Simulation a -> Simulation b
 {-# INLINE liftMS #-}
 liftMS f (Simulation x) =
@@ -163,6 +167,34 @@ finallySimulation (Simulation m) (Simulation m') =
 throwSimulation :: Exception e => e -> Simulation a
 throwSimulation = throw
 
+-- | Runs an action with asynchronous exceptions disabled.
+maskSimulation :: ((forall a. Simulation a -> Simulation a) -> Simulation b) -> Simulation b
+maskSimulation a =
+  Simulation $ \r ->
+  MC.mask $ \u ->
+  invokeSimulation r (a $ q u)
+  where q u (Simulation b) = Simulation (u . b)
+
+-- | Like 'maskSimulation', but the masked computation is not interruptible.
+uninterruptibleMaskSimulation :: ((forall a. Simulation a -> Simulation a) -> Simulation b) -> Simulation b
+uninterruptibleMaskSimulation a =
+  Simulation $ \r ->
+  MC.uninterruptibleMask $ \u ->
+  invokeSimulation r (a $ q u)
+  where q u (Simulation b) = Simulation (u . b)
+
+-- | An implementation of 'generalBracket'.
+generalBracketSimulation :: Simulation a
+                            -> (a -> MC.ExitCase b -> Simulation c)
+                            -> (a -> Simulation b)
+                            -> Simulation (b, c)
+generalBracketSimulation acquire release use =
+  Simulation $ \r -> do
+    MC.generalBracket
+      (invokeSimulation r acquire)
+      (\resource e -> invokeSimulation r $ release resource e)
+      (\resource -> invokeSimulation r $ use resource)
+
 -- | Invoke the 'Simulation' computation.
 invokeSimulation :: Run -> Simulation a -> IO a
 {-# INLINE invokeSimulation #-}
@@ -173,6 +205,17 @@ instance MonadFix Simulation where
     Simulation $ \r ->
     do { rec { a <- invokeSimulation r (f a) }; return a }  
 
+instance MC.MonadThrow Simulation where
+  throwM = throwSimulation
+
+instance MC.MonadCatch Simulation where
+  catch = catchSimulation
+
+instance MC.MonadMask Simulation where
+  mask = maskSimulation
+  uninterruptibleMask = uninterruptibleMaskSimulation
+  generalBracket = generalBracketSimulation
+
 -- | Memoize the 'Simulation' computation, always returning the same value
 -- within a simulation run.
 memoSimulation :: Simulation a -> Simulation (Simulation a)
