@@ -1,7 +1,7 @@
 
 -- |
 -- Module     : Simulation.Aivika.Internal.Specs
--- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2017, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -13,6 +13,7 @@ module Simulation.Aivika.Internal.Specs
         Method(..),
         Run(..),
         Point(..),
+        EventPriority(..),
         EventQueue(..),
         newEventQueue,
         basicTime,
@@ -27,12 +28,15 @@ module Simulation.Aivika.Internal.Specs
         integPointsStartingFrom,
         integStartPoint,
         integStopPoint,
-        pointAt) where
+        simulationStopPoint,
+        timeGrid,
+        pointAt,
+        delayPoint) where
 
 import Data.IORef
 
 import Simulation.Aivika.Generator
-import qualified Simulation.Aivika.PriorityQueue as PQ
+import qualified Simulation.Aivika.PriorityQueue.EventQueue as PQ
 
 -- | It defines the simulation specs.
 data Specs = Specs { spcStartTime :: Double,    -- ^ the start time
@@ -47,6 +51,7 @@ data Specs = Specs { spcStartTime :: Double,    -- ^ the start time
 data Method = Euler          -- ^ Euler's method
             | RungeKutta2    -- ^ the 2nd order Runge-Kutta method
             | RungeKutta4    -- ^ the 4th order Runge-Kutta method
+            | RungeKutta4b   -- ^ the 4th order Runge-Kutta 3/8-method
             deriving (Eq, Ord, Show)
 
 -- | It indentifies the simulation run.
@@ -61,10 +66,14 @@ data Run = Run { runSpecs :: Specs,  -- ^ the simulation specs
 data Point = Point { pointSpecs :: Specs,    -- ^ the simulation specs
                      pointRun :: Run,        -- ^ the simulation run
                      pointTime :: Double,    -- ^ the current time
+                     pointPriority :: EventPriority,    -- ^ the current priority
                      pointIteration :: Int,  -- ^ the current iteration
                      pointPhase :: Int       -- ^ the current phase
                    }
 
+-- | The event priority (greater is higher).
+type EventPriority = Int
+
 -- | It represents the event queue.
 data EventQueue = EventQueue { queuePQ :: PQ.PriorityQueue (Point -> IO ()),
                                -- ^ the underlying priority queue
@@ -132,6 +141,7 @@ integPhases sc =
     Euler -> [0]
     RungeKutta2 -> [0, 1]
     RungeKutta4 -> [0, 1, 2, 3]
+    RungeKutta4b -> [0, 1, 2, 3]
 
 -- | Returns the first and last integration phases.
 integPhaseBnds :: Specs -> (Int, Int)
@@ -140,6 +150,7 @@ integPhaseBnds sc =
     Euler -> (0, 0)
     RungeKutta2 -> (0, 1)
     RungeKutta4 -> (0, 3)
+    RungeKutta4b -> (0, 3)
 
 -- | Returns the first integration phase, i.e. zero.
 integPhaseLoBnd :: Specs -> Int
@@ -152,6 +163,7 @@ integPhaseHiBnd sc =
     Euler -> 0
     RungeKutta2 -> 1
     RungeKutta4 -> 3
+    RungeKutta4b -> 3
 
 -- | Returns a simulation time for the integration point specified by 
 -- the specs, iteration and phase.
@@ -169,6 +181,10 @@ basicTime sc n ph =
             delta RungeKutta4 1 = spcDT sc / 2
             delta RungeKutta4 2 = spcDT sc / 2
             delta RungeKutta4 3 = spcDT sc
+            delta RungeKutta4b 0 = 0
+            delta RungeKutta4b 1 = spcDT sc / 3
+            delta RungeKutta4b 2 = 2 * spcDT sc / 3
+            delta RungeKutta4b 3 = spcDT sc
 
 -- | Return the integration time values.
 integTimes :: Specs -> [Double]
@@ -185,6 +201,7 @@ integPoints r = points
         point n  = Point { pointSpecs = sc,
                            pointRun = r,
                            pointTime = basicTime sc n 0,
+                           pointPriority = 0,
                            pointIteration = n,
                            pointPhase = 0 }
 
@@ -196,10 +213,11 @@ integStartPoint r = point nl
         point n  = Point { pointSpecs = sc,
                            pointRun = r,
                            pointTime = basicTime sc n 0,
+                           pointPriority = 0,
                            pointIteration = n,
                            pointPhase = 0 }
 
--- | Return the stop time point.
+-- | Return the integration stop time point.
 integStopPoint :: Run -> Point
 integStopPoint r = point nu
   where sc = runSpecs r
@@ -207,12 +225,17 @@ integStopPoint r = point nu
         point n  = Point { pointSpecs = sc,
                            pointRun = r,
                            pointTime = basicTime sc n 0,
+                           pointPriority = 0,
                            pointIteration = n,
                            pointPhase = 0 }
 
--- | Return the point at the specified time.
-pointAt :: Run -> Double -> Point
-pointAt r t = p
+-- | Return the simulation stop time point.
+simulationStopPoint :: Run -> Point
+simulationStopPoint r = pointAt r (spcStopTime $ runSpecs r) 0
+
+-- | Return the point at the specified time given the priority.
+pointAt :: Run -> Double -> EventPriority -> Point
+pointAt r t priority = p
   where sc = runSpecs r
         t0 = spcStartTime sc
         dt = spcDT sc
@@ -220,10 +243,11 @@ pointAt r t = p
         p = Point { pointSpecs = sc,
                     pointRun = r,
                     pointTime = t,
+                    pointPriority = priority,
                     pointIteration = n,
                     pointPhase = -1 }
 
--- | Return the integration time points startin from the specified iteration.
+-- | Return the integration time points starting from the specified iteration.
 integPointsStartingFrom :: Point -> [Point]
 integPointsStartingFrom p = points
   where r  = pointRun p
@@ -236,5 +260,38 @@ integPointsStartingFrom p = points
         point n  = Point { pointSpecs = sc,
                            pointRun = r,
                            pointTime = basicTime sc n 0,
+                           pointPriority = 0,
                            pointIteration = n,
                            pointPhase = 0 }
+
+-- | Return the indexed time values in the grid by specified size.
+timeGrid :: Specs -> Int -> [(Int, Double)]
+timeGrid sc n =
+  let t0 = spcStartTime sc
+      t2 = spcStopTime sc
+      n' = max (n - 1) 1
+      dt = (t2 - t0) / (fromIntegral n')
+      f i | i == 0    = (i, t0)
+          | i == n'   = (i, t2)
+          | otherwise = (i, t0 + (fromIntegral i) * dt)
+  in map f [0 .. n']
+
+-- | Delay the point by the specified positive number of iterations.
+delayPoint :: Point -> Int -> Point
+delayPoint p dn
+  | dn <= 0   = error "Expected the positive number of iterations: delayPoint"
+  | otherwise =
+    let sc = pointSpecs p
+        n  = pointIteration p
+        ph = pointPhase p
+    in if ph < 0
+       then let t' = pointTime p - fromIntegral dn * spcDT sc
+                n' = fromIntegral $ floor $ (t' - spcStartTime sc) / spcDT sc
+            in p { pointTime = t',
+                   pointIteration = n',
+                   pointPhase = -1 }
+       else let n' = n - dn
+                t' = basicTime sc n' ph
+            in p { pointTime = t',
+                   pointIteration = n',
+                   pointPhase = ph }
