@@ -1,7 +1,7 @@
 
 -- |
 -- Module     : Simulation.Aivika.Signal
--- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2017, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -16,6 +16,7 @@ module Simulation.Aivika.Signal
        (-- * Handling and Triggering Signal
         Signal(..),
         handleSignal_,
+        handleSignalComposite,
         SignalSource,
         newSignalSource,
         publishSignal,
@@ -40,6 +41,7 @@ module Simulation.Aivika.Signal
         newSignalInIntegTimes,
         newSignalInStartTime,
         newSignalInStopTime,
+        newSignalInTimeGrid,
         -- * Delaying Signal
         delaySignal,
         delaySignalM,
@@ -58,7 +60,9 @@ module Simulation.Aivika.Signal
         traceSignal) where
 
 import Data.IORef
-import Data.Monoid
+import Data.Monoid hiding ((<>))
+import Data.Semigroup (Semigroup(..))
+import Data.List.NonEmpty (NonEmpty((:|)))
 import Data.List
 import Data.Array
 
@@ -68,8 +72,10 @@ import Control.Monad.Trans
 import Simulation.Aivika.Internal.Specs
 import Simulation.Aivika.Internal.Parameter
 import Simulation.Aivika.Internal.Simulation
+import Simulation.Aivika.Internal.Dynamics
 import Simulation.Aivika.Internal.Event
 import Simulation.Aivika.Internal.Arrival
+import Simulation.Aivika.Composite
 
 import qualified Simulation.Aivika.Vector as V
 import qualified Simulation.Aivika.Vector.Unboxed as UV
@@ -111,6 +117,12 @@ handleSignal_ :: Signal a -> (a -> Event ()) -> Event ()
 handleSignal_ signal h = 
   do x <- handleSignal signal h
      return ()
+
+-- | Like 'handleSignal' but within the 'Composite' computation.
+handleSignalComposite :: Signal a -> (a -> Event ()) -> Composite ()
+handleSignalComposite signal h =
+  do x <- liftEvent $ handleSignal signal h
+     disposableComposite x
      
 -- | Create a new signal source.
 newSignalSource :: Simulation (SignalSource a)
@@ -153,25 +165,30 @@ enqueueSignalHandler q h =
 dequeueSignalHandler :: SignalHandlerQueue a -> SignalHandler a -> IO ()
 {-# INLINE dequeueSignalHandler #-}
 dequeueSignalHandler q h = 
-  modifyIORef (queueList q) (delete h)
+  modifyIORef' (queueList q) $ delete h
 
 instance Functor Signal where
   fmap = mapSignal
-  
+
+instance Semigroup (Signal a) where
+  (<>) = merge2Signals
+
+  sconcat (x1 :| []) = x1
+  sconcat (x1 :| [x2]) = merge2Signals x1 x2
+  sconcat (x1 :| [x2, x3]) = merge3Signals x1 x2 x3
+  sconcat (x1 :| [x2, x3, x4]) = merge4Signals x1 x2 x3 x4
+  sconcat (x1 :| [x2, x3, x4, x5]) = merge5Signals x1 x2 x3 x4 x5
+  sconcat (x1 :| x2 : x3 : x4 : x5 : xs) =
+    sconcat $ merge5Signals x1 x2 x3 x4 x5 :| xs
+
 instance Monoid (Signal a) where 
   
   mempty = emptySignal
   
-  mappend = merge2Signals
+  mappend = (<>)
   
   mconcat [] = emptySignal
-  mconcat [x1] = x1
-  mconcat [x1, x2] = merge2Signals x1 x2
-  mconcat [x1, x2, x3] = merge3Signals x1 x2 x3
-  mconcat [x1, x2, x3, x4] = merge4Signals x1 x2 x3 x4
-  mconcat [x1, x2, x3, x4, x5] = merge5Signals x1 x2 x3 x4 x5
-  mconcat (x1 : x2 : x3 : x4 : x5 : xs) = 
-    mconcat $ merge5Signals x1 x2 x3 x4 x5 : xs
+  mconcat (h : t) = sconcat (h :| t)
   
 -- | Map the signal according the specified function.
 mapSignal :: (a -> b) -> Signal a -> Signal b
@@ -278,24 +295,24 @@ data SignalHistory a =
                   signalHistoryValues :: V.Vector a }
 
 -- | Create a history of the signal values.
-newSignalHistory :: Signal a -> Event (SignalHistory a)
+newSignalHistory :: Signal a -> Composite (SignalHistory a)
 newSignalHistory =
   newSignalHistoryStartingWith Nothing
 
 -- | Create a history of the signal values starting with
 -- the optional initial value.
-newSignalHistoryStartingWith :: Maybe a -> Signal a -> Event (SignalHistory a)
+newSignalHistoryStartingWith :: Maybe a -> Signal a -> Composite (SignalHistory a)
 newSignalHistoryStartingWith init signal =
-  Event $ \p ->
-  do ts <- UV.newVector
-     xs <- V.newVector
+  do ts <- liftIO UV.newVector
+     xs <- liftIO V.newVector
      case init of
        Nothing -> return ()
        Just a ->
-         do UV.appendVector ts (pointTime p)
-            V.appendVector xs a
-     invokeEvent p $
-       handleSignal_ signal $ \a ->
+         do t <- liftDynamics time
+            liftIO $
+              do UV.appendVector ts t
+                 V.appendVector xs a
+     handleSignalComposite signal $ \a ->
        Event $ \p ->
        do UV.appendVector ts (pointTime p)
           V.appendVector xs a
@@ -347,6 +364,18 @@ newSignalInStopTime =
      enqueueEvent t $ triggerSignalWithCurrentTime s
      return $ publishSignal s
 
+-- | Return a signal that is trigged in the grid by specified size.
+newSignalInTimeGrid :: Int -> Event (Signal Int)
+newSignalInTimeGrid n =
+  do sc <- liftParameter simulationSpecs
+     s  <- liftSimulation newSignalSource
+     let loop []            = return ()
+         loop ((i, t) : xs) = enqueueEvent t $
+                              do triggerSignal s i
+                                 loop xs
+     loop $ timeGrid sc n
+     return $ publishSignal s
+
 -- | Describes a computation that also signals when changing its value.
 data Signalable a =
   Signalable { readSignalable :: Event a,
@@ -363,10 +392,13 @@ signalableChanged x = mapSignalM (const $ readSignalable x) $ signalableChanged_
 instance Functor Signalable where
   fmap f x = x { readSignalable = fmap f (readSignalable x) }
 
-instance Monoid a => Monoid (Signalable a) where
+instance Semigroup a => Semigroup (Signalable a) where
+  (<>) = appendSignalable
+
+instance (Monoid a, Semigroup a) => Monoid (Signalable a) where
 
   mempty = emptySignalable
-  mappend = appendSignalable
+  mappend = (<>)
 
 -- | Return an identity.
 emptySignalable :: Monoid a => Signalable a
@@ -375,7 +407,7 @@ emptySignalable =
                signalableChanged_ = mempty }
 
 -- | An associative operation.
-appendSignalable :: Monoid a => Signalable a -> Signalable a -> Signalable a
+appendSignalable :: Semigroup a => Signalable a -> Signalable a -> Signalable a
 appendSignalable m1 m2 =
   Signalable { readSignalable = liftM2 (<>) (readSignalable m1) (readSignalable m2),
                signalableChanged_ = (signalableChanged_ m1) <> (signalableChanged_ m2) }
