@@ -1,7 +1,7 @@
 
 -- |
 -- Module     : Simulation.Aivika.Specs
--- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2017, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -20,6 +20,7 @@ module Simulation.Aivika.Specs
         integPhaseBnds,
         integPhaseHiBnd,
         integPhaseLoBnd,
-        integTimes) where
+        integTimes,
+        timeGrid) where
 
 import Simulation.Aivika.Internal.Specs
