@@ -1,7 +1,9 @@
 
+{-# LANGUAGE DeriveDataTypeable, DeriveGeneric #-}
+
 -- |
 -- Module     : Simulation.Aivika.Statistics
--- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2017, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -44,7 +46,14 @@ module Simulation.Aivika.Statistics
         setTimingCounter,
         returnTimingCounter) where
 
-import Data.Monoid
+import GHC.Generics (Generic)
+
+import Control.DeepSeq
+
+import Data.Monoid hiding ((<>))
+import Data.Semigroup (Semigroup(..))
+import Data.Typeable
+import Data.Binary
 
 -- | Defines data types that can be converted to 'Double'.
 class Ord a => ConvertableToDouble a where
@@ -69,11 +78,16 @@ data SamplingStats a =
                   -- ^ The maximum value among the samples.
                   samplingStatsMean  :: !Double,
                   -- ^ The average value.
-                  samplingStatsMean2 :: !Double 
+                  samplingStatsMean2 :: !Double, 
                   -- ^ The average square value.
+                  samplingStatsNormalised :: !Bool
+                  -- ^ Whether the statistics is normalised.
                 }
-  deriving (Eq, Ord)
-           
+  deriving (Eq, Ord, Typeable, Generic)
+
+instance NFData a => NFData (SamplingStats a)
+instance Binary a => Binary (SamplingStats a)
+
 -- | Specifies data type from which values we can gather the statistics.           
 class Num a => SamplingData a where           
   
@@ -86,11 +100,14 @@ class Num a => SamplingData a where
   -- | Combine two statistics.
   combineSamplingStats :: SamplingStats a -> SamplingStats a -> SamplingStats a
 
+instance SamplingData a => Semigroup (SamplingStats a) where
+  (<>) = combineSamplingStats
+
 instance SamplingData a => Monoid (SamplingStats a) where 
   
   mempty = emptySamplingStats
   
-  mappend = combineSamplingStats
+  mappend = (<>)
 
 instance SamplingData Double where
 
@@ -99,7 +116,8 @@ instance SamplingData Double where
                     samplingStatsMin   = 1 / 0,
                     samplingStatsMax   = (-1) / 0,
                     samplingStatsMean  = 0 / 0,
-                    samplingStatsMean2 = 0 / 0 }
+                    samplingStatsMean2 = 0 / 0,
+                    samplingStatsNormalised = False }
     
   addSamplingStats = addSamplingStatsGeneric
   
@@ -112,7 +130,8 @@ instance SamplingData Int where
                     samplingStatsMin   = maxBound,
                     samplingStatsMax   = minBound,
                     samplingStatsMean  = 0 / 0,
-                    samplingStatsMean2 = 0 / 0 }
+                    samplingStatsMean2 = 0 / 0,
+                    samplingStatsNormalised = False }
     
   addSamplingStats = addSamplingStatsGeneric
 
@@ -125,12 +144,14 @@ addSamplingStatsGeneric a stats
                                  samplingStatsMin   = a,
                                  samplingStatsMax   = a,
                                  samplingStatsMean  = x,
-                                 samplingStatsMean2 = x * x }
+                                 samplingStatsMean2 = x * x,
+                                 samplingStatsNormalised = normalised }
   | otherwise  = SamplingStats { samplingStatsCount = count,
                                  samplingStatsMin   = minX,
                                  samplingStatsMax   = maxX,
                                  samplingStatsMean  = meanX,
-                                 samplingStatsMean2 = meanX2 }
+                                 samplingStatsMean2 = meanX2,
+                                 samplingStatsNormalised = normalised }
     where count  = 1 + samplingStatsCount stats
           minX   = a `seq` min a (samplingStatsMin stats)
           maxX   = a `seq` max a (samplingStatsMax stats)
@@ -140,17 +161,19 @@ addSamplingStatsGeneric a stats
           x      = convertToDouble a
           k1     = 1.0 / n
           k2     = (n - 1.0) / n
+          normalised = samplingStatsNormalised stats
 
 combineSamplingStatsGeneric :: ConvertableToDouble a =>
                                SamplingStats a -> SamplingStats a -> SamplingStats a
 combineSamplingStatsGeneric stats1 stats2
-  | c1 == 0   = stats2
-  | c2 == 0   = stats1
+  | c1 == 0   = stats2 { samplingStatsNormalised = normalised }
+  | c2 == 0   = stats1 { samplingStatsNormalised = normalised }
   | otherwise = SamplingStats { samplingStatsCount = c,
                                 samplingStatsMin   = minZ,
                                 samplingStatsMax   = maxZ,
                                 samplingStatsMean  = meanZ,
-                                samplingStatsMean2 = meanZ2 }
+                                samplingStatsMean2 = meanZ2,
+                                samplingStatsNormalised = normalised }
   where c1     = samplingStatsCount stats1
         c2     = samplingStatsCount stats2
         c      = c1 + c2
@@ -171,6 +194,7 @@ combineSamplingStatsGeneric stats1 stats2
         meanZ2 = k1 * meanX2 + k2 * meanY2
         k1     = n1 / n
         k2     = n2 / n
+        normalised = samplingStatsNormalised stats1 || samplingStatsNormalised stats2
 
 -- | If allows combining statistics more efficiently if we know that the first argument can be a scalar.
 combineSamplingStatsEither :: SamplingData a => Either a (SamplingStats a) -> SamplingStats a -> SamplingStats a
@@ -180,8 +204,9 @@ combineSamplingStatsEither (Right stats1) stats2 = combineSamplingStats stats1 s
 -- | Return the variance.
 samplingStatsVariance :: SamplingStats a -> Double
 samplingStatsVariance stats
-  | count == 1 = 0
-  | otherwise  = (meanX2 - meanX * meanX) * (n / (n - 1))
+  | count == 1                    = 0
+  | samplingStatsNormalised stats = meanX2 - meanX * meanX
+  | otherwise                     = (meanX2 - meanX * meanX) * (n / (n - 1))
     where count  = samplingStatsCount stats
           meanX  = samplingStatsMean stats
           meanX2 = samplingStatsMean2 stats
@@ -213,6 +238,7 @@ showSamplingStats stats =
   showString ", std = " . shows (samplingStatsDeviation stats) . 
   showString ", min = " . shows (samplingStatsMin stats) . 
   showString ", max = " . shows (samplingStatsMax stats) .
+  (if samplingStatsNormalised stats then showString ", normalised = True" else id) .
   showString " }"
 
 instance Show a => Show (SamplingStats a) where
@@ -235,7 +261,12 @@ samplingStatsSummary stats indent =
      showString "min = " . shows (samplingStatsMin stats) . 
      showString "\n" .
      showString tab .
-     showString "max = " . shows (samplingStatsMax stats)
+     showString "max = " . shows (samplingStatsMax stats) .
+     (if samplingStatsNormalised stats
+       then showString "\n" .
+            showString tab .
+            showString "normalised = True"
+       else id)
      
 -- | This is the timing statistics where data are bound to the time.
 data TimingStats a =
@@ -259,7 +290,10 @@ data TimingStats a =
                 -- ^ Return the sum of values.
                 timingStatsSum2      :: !Double 
                 -- ^ Return the sum of square values.
-                } deriving (Eq, Ord)
+                } deriving (Eq, Ord, Typeable, Generic)
+
+instance NFData a => NFData (TimingStats a)
+instance Binary a => Binary (TimingStats a)
                            
 -- | Defines the data type from which values we can gather the timing statistics.
 class Num a => TimingData a where                           
@@ -410,7 +444,8 @@ normTimingStats n stats =
                   samplingStatsMin   = timingStatsMin stats,
                   samplingStatsMax   = timingStatsMax stats,
                   samplingStatsMean  = timingStatsMean stats,
-                  samplingStatsMean2 = timingStatsMean2 stats }
+                  samplingStatsMean2 = timingStatsMean2 stats,
+                  samplingStatsNormalised = True }
 
 -- | Show the summary of the statistics.       
 showTimingStats :: (Show a, TimingData a) => TimingStats a -> ShowS
@@ -461,7 +496,10 @@ data SamplingCounter a =
                     -- ^ The counter value.
                     samplingCounterStats :: SamplingStats a
                     -- ^ The counter statistics.
-                  } deriving (Eq, Ord, Show)
+                  } deriving (Eq, Ord, Show, Typeable, Generic)
+
+instance NFData a => NFData (SamplingCounter a)
+instance Binary a => Binary (SamplingCounter a)
 
 -- | An empty counter.
 emptySamplingCounter :: SamplingData a => SamplingCounter a
@@ -501,7 +539,10 @@ data TimingCounter a =
                   -- ^ The counter value.
                   timingCounterStats :: TimingStats a
                   -- ^ The counter statistics.
-                } deriving (Eq, Ord, Show)
+                } deriving (Eq, Ord, Show, Typeable, Generic)
+
+instance NFData a => NFData (TimingCounter a)
+instance Binary a => Binary (TimingCounter a)
 
 -- | An empty counter.
 emptyTimingCounter :: TimingData a => TimingCounter a
