@@ -1,7 +1,7 @@
 
 -- |
 -- Module     : Simulation.Aivika.Stream
--- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2017, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -83,7 +83,9 @@ module Simulation.Aivika.Stream
 
 import Data.IORef
 import Data.Maybe
-import Data.Monoid
+import Data.Monoid hiding ((<>))
+import Data.Semigroup (Semigroup(..))
+import Data.List.NonEmpty (NonEmpty((:|)))
 
 import Control.Applicative
 import Control.Monad
@@ -123,11 +125,17 @@ instance Alternative Stream where
 
   (<|>) = mergeStreams
 
+instance Semigroup (Stream a) where
+
+  (<>) = mergeStreams
+
+  sconcat (h :| t) = concatStreams (h : t)
+
 instance Monoid (Stream a) where
 
   mempty  = emptyStream
 
-  mappend = mergeStreams
+  mappend = (<>)
 
   mconcat = concatStreams
 
@@ -281,7 +289,7 @@ rightStream (Cons s) = Cons y where
 
 -- | Replace the 'Left' values.
 replaceLeftStream :: Stream (Either a b) -> Stream c -> Stream (Either c b)
-replaceLeftStream (Cons sab) (ys0 @ ~(Cons sc)) = Cons z where
+replaceLeftStream (Cons sab) (ys0@(Cons sc)) = Cons z where
   z = do (a, xs) <- sab
          case a of
            Left _ ->
@@ -292,7 +300,7 @@ replaceLeftStream (Cons sab) (ys0 @ ~(Cons sc)) = Cons z where
 
 -- | Replace the 'Right' values.
 replaceRightStream :: Stream (Either a b) -> Stream c -> Stream (Either a c)
-replaceRightStream (Cons sab) (ys0 @ ~(Cons sc)) = Cons z where
+replaceRightStream (Cons sab) (ys0@(Cons sc)) = Cons z where
   z = do (a, xs) <- sab
          case a of
            Right _ ->
