@@ -3,7 +3,7 @@
 
 -- |
 -- Module     : Simulation.Aivika.SystemDynamics
--- Copyright  : Copyright (c) 2009-2016, David Sorokin <david.sorokin@gmail.com>
+-- Copyright  : Copyright (c) 2009-2017, David Sorokin <david.sorokin@gmail.com>
 -- License    : BSD3
 -- Maintainer : David Sorokin <david.sorokin@gmail.com>
 -- Stability  : experimental
@@ -49,6 +49,8 @@ module Simulation.Aivika.SystemDynamics
         -- * Discrete Functions
         delay,
         delayI,
+        delayByDT,
+        delayIByDT,
         step,
         pulse,
         pulseP,
@@ -237,6 +239,79 @@ integRK4 (Dynamics f) (Dynamics i) (Dynamics y) p =
     _ -> 
       error "Incorrect phase: integRK4"
 
+integRK4b :: Dynamics Double
+             -> Dynamics Double
+             -> Dynamics Double
+             -> Point -> IO Double
+integRK4b (Dynamics f) (Dynamics i) (Dynamics y) p =
+  case pointPhase p of
+    0 -> case pointIteration p of
+      0 -> 
+        i p
+      n -> do
+        let sc = pointSpecs p
+            ty = basicTime sc (n - 1) 0
+            t1 = ty
+            t2 = basicTime sc (n - 1) 1
+            t3 = basicTime sc (n - 1) 2
+            t4 = basicTime sc (n - 1) 3
+            py = p { pointTime = ty, pointIteration = n - 1, pointPhase = 0 }
+            p1 = py
+            p2 = p { pointTime = t2, pointIteration = n - 1, pointPhase = 1 }
+            p3 = p { pointTime = t3, pointIteration = n - 1, pointPhase = 2 }
+            p4 = p { pointTime = t4, pointIteration = n - 1, pointPhase = 3 }
+        vy <- y py
+        k1 <- f p1
+        k2 <- f p2
+        k3 <- f p3
+        k4 <- f p4
+        let !v = vy + spcDT sc / 8.0 * (k1 + 3.0 * (k2 + k3) + k4)
+        return v
+    1 -> do
+      let sc = pointSpecs p
+          n  = pointIteration p
+          ty = basicTime sc n 0
+          t1 = ty
+          py = p { pointTime = ty, pointIteration = n, pointPhase = 0 }
+          p1 = py
+      vy <- y py
+      k1 <- f p1
+      let !v = vy + spcDT sc / 3.0 * k1
+      return v
+    2 -> do
+      let sc = pointSpecs p
+          n  = pointIteration p
+          ty = basicTime sc n 0
+          t1 = ty
+          t2 = basicTime sc n 1
+          py = p { pointTime = ty, pointIteration = n, pointPhase = 0 }
+          p1 = py
+          p2 = p { pointTime = t2, pointIteration = n, pointPhase = 1 }
+      vy <- y py
+      k1 <- f p1
+      k2 <- f p2
+      let !v = vy + spcDT sc * (- k1 / 3.0 + k2)
+      return v
+    3 -> do
+      let sc = pointSpecs p
+          n  = pointIteration p
+          ty = basicTime sc n 0
+          t1 = ty
+          t2 = basicTime sc n 1
+          t3 = basicTime sc n 2
+          py = p { pointTime = ty, pointIteration = n, pointPhase = 0 }
+          p1 = py
+          p2 = p { pointTime = t2, pointIteration = n, pointPhase = 1 }
+          p3 = p { pointTime = t3, pointIteration = n, pointPhase = 2 }
+      vy <- y py
+      k1 <- f p1
+      k2 <- f p2
+      k3 <- f p3
+      let !v = vy + spcDT sc * (k1 - k2 + k3)
+      return v
+    _ -> 
+      error "Incorrect phase: integRK4b"
+
 -- | Return an integral with the specified derivative and initial value.
 --
 -- To create a loopback, you should use the recursive do-notation.
@@ -263,6 +338,7 @@ integ diff i =
           Euler -> return $ Dynamics $ integEuler diff i y
           RungeKutta2 -> return $ Dynamics $ integRK2 diff i y
           RungeKutta4 -> return $ Dynamics $ integRK4 diff i y
+          RungeKutta4b -> return $ Dynamics $ integRK4b diff i y
       return y
 
 integEulerEither :: Dynamics (Either Double Double)
@@ -584,6 +660,7 @@ lookupStepwiseDynamics (Dynamics m) tbl =
 --
 
 -- | Return the delayed value using the specified lag time.
+-- This function is less accurate than 'delayByDT'.
 delay :: Dynamics a          -- ^ the value to delay
          -> Dynamics Double  -- ^ the lag time
          -> Dynamics a       -- ^ the delayed value
@@ -612,6 +689,7 @@ delay (Dynamics x) (Dynamics d) = discreteDynamics $ Dynamics r
 
 -- | Return the delayed value using the specified lag time and initial value.
 -- Because of the latter, it allows creating a loop back.
+-- This function is less accurate than 'delayIByDT'.
 delayI :: Dynamics a          -- ^ the value to delay
           -> Dynamics Double  -- ^ the lag time
           -> Dynamics a       -- ^ the initial value
@@ -632,10 +710,71 @@ delayI (Dynamics x) (Dynamics d) (Dynamics i) = M.memo0Dynamics $ Dynamics r
                                   pointIteration = n',
                                   pointPhase = -1 }
             | n' > n    = error $
-                          "Cannot return the future data: delay. " ++
+                          "Cannot return the future data: delayI. " ++
                           "The lag time cannot be negative."
             | otherwise = error $
-                          "Cannot return the current data: delay. " ++
+                          "Cannot return the current data: delayI. " ++
+                          "The lag time is too small."
+      y
+
+-- | Return the delayed value by the specified positive number of
+-- integration time steps used for calculating the lag time.
+delayByDT :: Dynamics a
+             -- ^ the value to delay
+             -> Dynamics Int
+             -- ^ the delay as a multiplication of the corresponding number
+             -- and the integration time step
+             -> Dynamics a
+             -- ^ the delayed value
+delayByDT (Dynamics x) (Dynamics d) = discreteDynamics $ Dynamics r 
+  where
+    r p = do 
+      let sc = pointSpecs p
+          n  = pointIteration p
+      a <- d p
+      let p' = delayPoint p a
+          n' = pointIteration p'
+          y | n' < 0    = x $ p { pointTime = spcStartTime sc,
+                                  pointIteration = 0, 
+                                  pointPhase = 0 }
+            | n' < n    = x p'
+            | n' > n    = error $
+                          "Cannot return the future data: delayByDT. " ++
+                          "The lag time cannot be negative."
+            | otherwise = error $
+                          "Cannot return the current data: delayByDT. " ++
+                          "The lag time is too small."
+      y
+      
+-- | Return the delayed value by the specified initial value and
+-- a positive number of integration time steps used for calculating
+-- the lag time. It allows creating a loop back.
+delayIByDT :: Dynamics a
+              -- ^ the value to delay
+              -> Dynamics Int
+              -- ^ the delay as a multiplication of the corresponding number
+              -- and the integration time step
+              -> Dynamics a
+              -- ^ the initial value
+              -> Simulation (Dynamics a)
+              -- ^ the delayed value
+delayIByDT (Dynamics x) (Dynamics d) (Dynamics i) = M.memoDynamics $ Dynamics r 
+  where
+    r p = do 
+      let sc = pointSpecs p
+          n  = pointIteration p
+      a <- d p
+      let p' = delayPoint p a
+          n' = pointIteration p'
+          y | n' < 0    = i $ p { pointTime = spcStartTime sc,
+                                  pointIteration = 0, 
+                                  pointPhase = 0 }
+            | n' < n    = x p'
+            | n' > n    = error $
+                          "Cannot return the future data: delayIByDT. " ++
+                          "The lag time cannot be negative."
+            | otherwise = error $
+                          "Cannot return the current data: delayIByDT. " ++
                           "The lag time is too small."
       y
 
