@@ -1,11 +1,12 @@
 
 {-# LANGUAGE RecursiveDo #-}
 
+-- This is model Chemical Reaction from the 5-minute tutorial of 
+-- Berkeley-Madonna.
+
 import Simulation.Aivika
 import Simulation.Aivika.SystemDynamics
 
-import qualified Data.Vector as V
-
 specs = Specs { spcStartTime = 0, 
                 spcStopTime = 13, 
                 spcDT = 0.01,
